// Adopted from https://github.com/motdotla/dotenv BSD-2
import { Logger } from "../common/log.js";
import fs from "fs";
import path from "path";
const log = Logger("zeed:env");
const NEWLINE = "\n";
const RE_INI_KEY_VAL = /^\s*([\w_.-]+)\s*=\s*(.*)?\s*$/;
const RE_NEWLINES = /\\n/g;
const NEWLINES_MATCH = /\n|\r|\r\n/;
// Parses src into an Object
function parse(src, options = {}) {
    let obj = {};
    // convert Buffers before splitting into lines and processing
    src
        .toString()
        .split(NEWLINES_MATCH)
        .forEach(function (line, idx) {
        // matching "KEY' and 'VAL' in 'KEY=VAL'
        const keyValueArr = line.match(RE_INI_KEY_VAL);
        // matched?
        // log.debug("keyValueArr", keyValueArr)
        if (keyValueArr != null) {
            const key = keyValueArr[1];
            // default undefined or missing values to empty string
            let val = keyValueArr[2] || "";
            const end = val.length - 1;
            const isDoubleQuoted = val[0] === '"' && val[end] === '"';
            const isSingleQuoted = val[0] === "'" && val[end] === "'";
            // if single or double quoted, remove quotes
            if (isSingleQuoted || isDoubleQuoted) {
                val = val.substring(1, end);
                // if double quoted, expand newlines
                if (isDoubleQuoted) {
                    val = val.replace(RE_NEWLINES, NEWLINE);
                }
            }
            else {
                // remove surrounding whitespace
                val = val.trim();
            }
            obj[key] = val;
        }
        else {
            log.debug(`did not match key and value when parsing line ${idx + 1}: ${line}`);
        }
    });
    // log.debug("obj", obj)
    return obj;
}
export function stringToPath(value) {
    return path.resolve(process.cwd(), value || ".");
}
// Populates process.env from .env file
export function setupEnv(options = {}) {
    var _a, _b, _c, _d;
    const dotenvPath = (_a = options === null || options === void 0 ? void 0 : options.path) !== null && _a !== void 0 ? _a : path.resolve(process.cwd(), (_b = options === null || options === void 0 ? void 0 : options.filename) !== null && _b !== void 0 ? _b : ".env");
    const encoding = (_c = options === null || options === void 0 ? void 0 : options.encoding) !== null && _c !== void 0 ? _c : "utf8";
    const debug = (options === null || options === void 0 ? void 0 : options.debug) || false;
    log.active = debug;
    try {
        // specifying an encoding returns a string instead of a buffer
        const parsedEnv = fs.existsSync(dotenvPath)
            ? parse(fs.readFileSync(dotenvPath, { encoding }), { debug })
            : {};
        const parsedEnvLocal = fs.existsSync(dotenvPath + ".local")
            ? parse(fs.readFileSync(dotenvPath + ".local", { encoding }), { debug })
            : {};
        const parsed = Object.assign({}, parsedEnv, parsedEnvLocal);
        let env = (_d = options === null || options === void 0 ? void 0 : options.env) !== null && _d !== void 0 ? _d : process.env;
        Object.entries(parsed).forEach(([key, value]) => {
            if (typeof (options === null || options === void 0 ? void 0 : options.prefix) === "string") {
                key = (options === null || options === void 0 ? void 0 : options.prefix) + key;
            }
            if (!Object.prototype.hasOwnProperty.call(env, key)) {
                if (value != null) {
                    log.info(`set env.${key} = ${value}`);
                    env[key] = value;
                }
            }
            else {
                log.debug(`"${key}" is already defined and will not be overwritten`);
            }
        });
        return { parsed };
    }
    catch (e) {
        log.error(e);
        return { error: e };
    }
}
//# sourceMappingURL=env.js.map